/******************************************************************************
 * Player.java - Describe one FIBS player
 * $Id: Player.java,v 1.11 2009/02/24 06:58:06 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright 2008 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Player.java,v $
 * Revision 1.11  2009/02/24 06:58:06  dick
 * Add isPlaying().
 *
 * Revision 1.10  2009/02/20 10:26:26  dick
 * Add id, which is a database key.
 *
 * Revision 1.9  2009/02/14 15:43:01  dick
 * BuckoFIBS is released under the GNU license.
 * Javadoc.
 *
 * Revision 1.8  2009/02/11 09:09:13  dick
 * setName(s) for when we get an invite from an unknown player.
 *
 * Revision 1.7  2009/02/01 21:14:23  dick
 * Add the MissManners attribute.
 *
 * Revision 1.6  2009/02/01 09:00:08  dick
 * Add a saved match status string.
 *
 * Revision 1.5  2009/01/28 22:10:57  dick
 * Wacky parsing error?  How ss[1] be null?
 *
 * Revision 1.4  2009/01/28 02:52:31  dick
 * Javadoc.
 *
 * Revision 1.3  2009/01/27 19:15:24  dick
 * Add bfStatus stuff to handle player warnings.
 *
 * Revision 1.2  2008/12/11 10:01:28  dick
 * Parse opponent named "-" as empty (no opponent).
 *
 * Revision 1.1  2008/03/31 07:10:56  dick
 * Describe one FIBS player.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.domain;


/** Describe one FIBS player. <br>
 * Most of these fields don't have setters because this object is mostly filled out from FIBS.
 * See {@link #parsePlayer(String)}.
 * @author Dick Balaska
 * @since 2008/03/31
 * @version $Revision: 1.11 $ <br> $Date: 2009/02/24 06:58:06 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/domain/Player.java">cvs Player.java</a>
 */
public class Player implements Comparable<Player> {
	private	int		id;
	private	String	name;
	private	String	opponent = "";
	private	String	watching = "";
	private	boolean	ready;
	private	boolean	away;
	private	double	rating;
	private	int		experience;
	private	int		idleTime;
	private	long	loginTime;
	private	String	hostName;
	private	String	client;
	private	String	email;

	private	int		bfFlag = 0;
	private	String	bfStatus;
	
//	private	boolean	hasSavedGame = false;
	private	String	savedMatch;
	private	String	missManners;

	/** Create a default/empty Player
	 */
	public Player() {}
	
	/** Get the local player id.
	 * This is a key to our local database, and is not a FIBS number at all.
	 * @return the id
	 */
	public int getId() {
		return id;
	}

	/** Set the local player id.
	 * This is a key to our local database, and is not a FIBS number at all.
	 * @param id the id to set
	 */
	public void setId(int id) {
		this.id = id;
	}

	/** Get the username of this player
	 * @return The username
	 */
	public String	getName() {
		return(name);
	}

	/** Set the username of this player.
	 * This is useful when we get an invite from an unknown player.
	 * @param name The player's name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/** Return true if this player is playing someone.
	 * This field is derived from the opponent field.
	 * @return true if playing.
	 */
	public boolean isPlaying() {
		if (opponent.length() > 0)
			return(true);
		return(false);
	}

	/** If this player is playing someone, then return the opponent's name.
	 * @return The opponent's name or the empty string if not playing anyone.
	 */
	public	String getOpponent() {
		return(opponent);
	}

	/** If this player is watching someone, then return the name of the player we are watching.
	 * @return The player being watched or the empty string if not watching anyone.
	 */
	public	String	getWatching() {
		return(watching);
	}

	/** Is this player ready to play?
	 * @return true if ready to play.
	 */
	public	boolean isReady() {
		return(ready);
	}

	/** Has this player marked himself as away?
	 * @return true = yes
	 */
	public	boolean	isAway() {
		return(away);
	}

	/** Get the rating for this player
	 * @return The rating
	 */
	public	double	getRating() {
		return(rating);
	}

	/** Get the number of games this player has played
	 * @return The Experience
	 */
	public int getExperience() {
		return(experience);
	}

	/** How long has this player been idle?
	 * @return The number of seconds he has not done anything.
	 */
	public int getIdleTime() {
		return(idleTime);
	}

	/** When did this player log in?
	 * @return The number of seconds since Jan. 1, 1970
	 */
	public long getLoginTime() {
		return(loginTime);
	}

	/** Get the hostname of the machine this player is playing on.
	 * @return The hostname
	 */
	public String getHostName() {
		return(hostName);
	}
	
	/** What client is this player using?
	 * @return The name of the client.
	 */
	public String getClient() {
		return(client);
	}

	/** Set the name of the client that this player is using.
	 * @param client
	 */
	public void setClient(String client) {
		this.client = client;
	}

	/** What is this player's email address?
	 * @return The email address that the player gave to FIBS (probably not a real email address).
	 */
	public String getEmail() {
		return(email);
	}

	/** Get the BuckoFIBS flag (not used yet)
	 * @return the bfFlag
	 */
	public int getBfFlag() {
		return bfFlag;
	}

	/** Set the BuckoFIBS flag (not used yet)
	 * @param bfFlag the bfFlag to set
	 */
	public void setBfFlag(int bfFlag) {
		this.bfFlag = bfFlag;
	}

	/** If this player has a special status string, like an invite warning, it goes here.
	 * @return the bfStatus The status string.
	 */
	public String getBfStatus() {
		return bfStatus;
	}

	/** If this player has a special status string, like an invite warning, it goes here.
	 * @param bfStatus the bfStatus to set
	 */
	public void setBfStatus(String bfStatus) {
		this.bfStatus = bfStatus;
	}

	/** If you have a saved match with this player, then this string describes it. 
	 * @return the savedMatch or null if none
	 */
	public String getSavedMatch() {
		return savedMatch;
	}

	/** Set the savedMatch (tooltip) text
	 * @param savedMatch the savedMatch to set
	 */
	public void setSavedMatch(String savedMatch) {
		this.savedMatch = savedMatch;
	}

	/** If this player has a MissManners warning, this is it.
	 * @return the missManners warning, or null if this player has none.
	 */
	public String getMissManners() {
		return missManners;
	}

	/** Set the MissManners warning issued about this player
	 * @param missManners the missManners to set
	 */
	public void setMissManners(String missManners) {
		this.missManners = missManners;
	}

	/** Parse this player from a FIBS string.
	 * Fills in most of this object's fields from the FIBS 
	 * <a href="http://www.fibs.com/fibs_interface.html#clip_who_info">who info</a> line.
	 * @param s The who info line from FIBS
	 * @return success
	 */
	public boolean parsePlayer(String s) {
		String[] ss = s.split(" ");
		if (ss.length != 13) {
			System.out.println("Error parsing player");
			System.out.println(s);
			return(false);
		}
		name = ss[1];
		opponent = ss[2];
		if (ss[2].equals("-") || opponent == null)
			opponent = "";
		watching = ss[3];
		if (watching.equals("-"))
			watching = "";
		ready = ss[4].equals("1") ? true : false;
		away = ss[5].equals("1") ? true : false;
		rating = Double.parseDouble(ss[6]);
		experience = Integer.parseInt(ss[7]);
		idleTime = Integer.parseInt(ss[8]);
		loginTime = Long.parseLong(ss[9]);
		hostName = ss[10];
		client = ss[11];
		email = ss[12];
		return(true);
	}

	/** String compare the name of this player with that one.
	 * @param o The other player to compare
	 */
	@Override
	public int compareTo(Player o) {
		return(this.name.compareToIgnoreCase(o.getName()));
	}
}
